# -*- coding: utf-8 -*-

# Authors: Alejandro J. Cura <alecu@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""The user interface for the control panel for Ubuntu One."""

from __future__ import division

import os

from PyQt4 import QtGui, QtCore
from twisted.internet import defer

from ubuntuone.controlpanel import backend
from ubuntuone.controlpanel.logger import setup_logging, log_call
from ubuntuone.controlpanel.gui import (
    ALWAYS_SUBSCRIBED,
    CONTACT_ICON_NAME,
    FREE_SPACE_TEXT,
    FILE_URI_PREFIX,
    FOLDER_ICON_NAME,
    FOLDERS_CONFIRM_MERGE,
    MUSIC_ICON_NAME,
    MUSIC_DISPLAY_NAME,
    MUSIC_REAL_PATH,
    MY_FOLDERS,
    NAME_NOT_SET,
    ORANGE,
    SHARE_ICON_NAME,
    SHARES_MIN_SIZE_FULL,
)
from ubuntuone.controlpanel.gui.qt.ui import folders_ui


logger = setup_logging('qt.folders')

FREE_SPACE = u'<font color="grey">%s</font>' % FREE_SPACE_TEXT
NO_FREE_SPACE = u'<font color="red"><b>%s</b></font>' % FREE_SPACE_TEXT
ROW_HEADER = u'<font size="5"><b>%s</b></font> %s'
ROOT = u'%s - <font color="%s" size="2">%s</font>'

FOLDER_NAME_COL = 0
SUBSCRIPTION_COL = 1

CANCEL = QtGui.QMessageBox.Cancel
CHECKED = QtCore.Qt.Checked
NO = QtGui.QMessageBox.No
UNCHECKED = QtCore.Qt.Unchecked
YES = QtGui.QMessageBox.Yes


@log_call(logger.debug)
def uri_hook(uri):
    """Open an URI using the default browser/file manager."""
    url = QtCore.QString(uri)
    QtGui.QDesktopServices.openUrl(QtCore.QUrl(url))


class FoldersPanel(QtGui.QWidget):
    """The Folders Tab Panel widget"""

    def __init__(self, *args):
        """Initialize the UI of the widget."""
        QtGui.QWidget.__init__(self, *args)
        self.ui = folders_ui.Ui_Form()
        self.ui.setupUi(self)

        self.is_processing = False
        self.backend = backend.ControlBackend()
        logger.debug('%s: started.', self.__class__.__name__)

    # Invalid name "showEvent"
    # pylint: disable=C0103

    def showEvent(self, event):
        """Load info."""
        self.is_processing = True
        headers = self.ui.folders.header()
        headers.setResizeMode(FOLDER_NAME_COL, headers.Stretch)
        headers.setResizeMode(SUBSCRIPTION_COL, headers.ResizeToContents)

        d = defer.maybeDeferred(self.backend.volumes_info)
        d.addCallback(self.process_folders)
        event.accept()

    def humanize(self, int_bytes):
        """Return a human readable representation of 'int_bytes'.

        This method follows the https://wiki.ubuntu.com/UnitsPolicy to build
        the result.

        """
        units = {0: 'bytes', 1: 'KiB', 2: 'MiB', 3: 'GiB', 4: 'TiB',
                 5: 'PiB', 6: 'Eib', 7: 'ZiB', 8: 'YiB'}
        unit = 0
        while int_bytes >= 1024:
            int_bytes = int_bytes / 1024
            unit += 1
        return '%.1f %s' % (int_bytes, units[unit])

    def _process_name(self, name):
        """Tweak 'name' with a translatable music folder name."""
        if name == MUSIC_REAL_PATH:
            result = MUSIC_DISPLAY_NAME
        else:
            result = name
        return result

    @log_call(logger.debug)
    def process_folders(self, info):
        """Load folders info into the tree view."""
        self.ui.folders.clear()

        if not info:
            return

        for name, free_bytes, volumes in info:
            if self.backend.NAME_NOT_SET in name:
                name = NAME_NOT_SET

            if name:
                name = name + "'s"
                # we already added user folders, let's add an empty row
                empty_row = QtGui.QTreeWidgetItem()
                empty_row.is_empty = True
                self.ui.folders.addTopLevelItem(empty_row)
            else:
                name = MY_FOLDERS

            if free_bytes == self.backend.FREE_BYTES_NOT_AVAILABLE:
                free_bytes = ''
            else:
                free_bytes = int(free_bytes)
                if free_bytes < SHARES_MIN_SIZE_FULL:
                    free_bytes_str = NO_FREE_SPACE
                else:
                    free_bytes_str = FREE_SPACE
                free_bytes_args = {'free_space': self.humanize(free_bytes)}
                free_bytes = free_bytes_str % free_bytes_args

            item = QtGui.QTreeWidgetItem()
            item.is_empty = False
            item.setIcon(FOLDER_NAME_COL,
                         QtGui.QIcon.fromTheme(CONTACT_ICON_NAME))
            self.ui.folders.addTopLevelItem(item)

            model_index = self.ui.folders.indexFromItem(item, FOLDER_NAME_COL)
            title = QtGui.QLabel(ROW_HEADER % (name, free_bytes))
            self.ui.folders.setIndexWidget(model_index, title)

            for volume in volumes:
                child = QtGui.QTreeWidgetItem()
                child.volume_path = volume['path']
                child.volume_id = volume['volume_id']

                name = self._process_name(volume[u'display_name'])
                icon_name = FOLDER_ICON_NAME

                is_root = volume[u'type'] == self.backend.ROOT_TYPE
                is_share = volume[u'type'] == self.backend.SHARE_TYPE

                if is_root:
                    name = ROOT % (name, ORANGE, ALWAYS_SUBSCRIBED)
                    flags = QtCore.Qt.ItemIsSelectable | \
                            QtCore.Qt.ItemIsEnabled
                    child.setFlags(flags)
                elif is_share:
                    icon_name = SHARE_ICON_NAME
                elif name == MUSIC_DISPLAY_NAME:
                    icon_name = MUSIC_ICON_NAME

                icon = QtGui.QIcon.fromTheme(icon_name)
                icon.icon_name = icon_name
                if bool(volume[u'subscribed']):
                    child.setCheckState(SUBSCRIPTION_COL, CHECKED)
                else:
                    child.setCheckState(SUBSCRIPTION_COL, UNCHECKED)
                    pixmap = icon.pixmap(24, icon.Disabled, icon.Off)
                    icon = QtGui.QIcon(pixmap)
                    icon.icon_name = icon_name

                child.icon_obj = icon  # hack!
                child.setIcon(FOLDER_NAME_COL, icon)
                item.addChild(child)

                model_index = self.ui.folders.indexFromItem(child,
                                                            FOLDER_NAME_COL)
                self.ui.folders.setIndexWidget(model_index, QtGui.QLabel(name))

        self.ui.folders.expandAll()
        self.is_processing = False

    def on_folders_itemActivated(self, item, column=None):
        """User activated a given row, open the path in a file browser."""
        volume_path = item.volume_path
        if volume_path is None:
            logger.warning('on_folders_itemActivated: volume_path for item %r '
                           'and column %r is None.', item, column)
        elif not os.path.exists(volume_path):
            logger.warning('on_folders_itemActivated: stored path %r '
                           'does not exist.', volume_path)
        else:
            uri = FILE_URI_PREFIX + volume_path
            uri_hook(uri)

    @defer.inlineCallbacks
    def on_folders_itemChanged(self, item, column=None):
        """User changed the subscription for a given folder."""
        volume_id = item.volume_id
        volume_path = item.volume_path
        if volume_id is None:
            logger.warning('on_folders_itemChanged: volume_id for item %r '
                           'and column %r is None.', item, column)
            return

        if self.is_processing:
            # ignore signals when the UI is being updated
            return

        subscribed = item.checkState(SUBSCRIPTION_COL) == CHECKED  # new state
        logger.info('on_folders_itemChanged: processing volume id %r with '
                    'path %r, new subscribed value is: %r. Path exists? %r',
                    volume_id, volume_path, subscribed,
                    os.path.exists(volume_path))
        response = YES
        if subscribed and os.path.exists(volume_path):
            text = FOLDERS_CONFIRM_MERGE % {'folder_path': volume_path}
            buttons = YES | NO | CANCEL
            response = QtGui.QMessageBox.warning(self, '', text, buttons, YES)

        self.is_processing = True

        if response == YES:
            # user accepted, merge the folder content
            yield self.backend.change_volume_settings(volume_id,
                                                {'subscribed': subscribed})
        else:
            # restore old value
            old = UNCHECKED if subscribed else CHECKED
            item.setCheckState(SUBSCRIPTION_COL, old)

        self.is_processing = False
