# -*- coding: utf-8 -*-

# Authors: Alejandro J. Cura <alecu@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""The user interface for the control panel for Ubuntu One."""

import sys

from dbus.mainloop.qt import DBusQtMainLoop
from PyQt4 import Qt, QtGui

from ubuntuone.controlpanel.logger import setup_logging
from ubuntuone.controlpanel.gui.qt.ui import mainwindow_ui


logger = setup_logging('qt.gui')


class MainWindow(QtGui.QMainWindow):
    """The Main Window of the Control Panel."""

    def __init__(self, close_callback=None):
        """Initialize this instance with the UI layout."""
        QtGui.QMainWindow.__init__(self)
        self.ui = mainwindow_ui.Ui_MainWindow()
        self.ui.setupUi(self)
        self.close_callback = close_callback

    # Invalid name "closeEvent"
    # pylint: disable=C0103

    def closeEvent(self, event):
        """The main window is being closed, call any custom callback."""
        if self.close_callback is not None:
            self.close_callback()
        event.accept()


def main(switch_to='', alert=False):
    """Start the Qt reactor and open the main window."""

    # The DBus main loop MUST be initialized before importing the reactor
    DBusQtMainLoop(set_as_default=True)

    # The app variable should be in scope of the reactor.run()
    # pylint: disable=W0612
    app = Qt.QApplication(sys.argv)

    # The following cannot be imported outside this function
    # because u1trial already provides a reactor.
    # pylint: disable=W0404, F0401
    from qtreactor import qt4reactor
    qt4reactor.install()
    from twisted.internet import reactor

    # Due to the crash with MemoryError as per
    # http://pastebin.ubuntu.com/612270/
    # we need patch SyncDaemonTool so there is no Dbus interaction
    from ubuntuone.controlpanel.integrationtests.test_sd_client.test_linux \
    import FakedSyncDaemonTool

    import os
    import subprocess

    class SDTool(FakedSyncDaemonTool):
        """A custom SDTool."""

        shares = [
            {u'accepted': u'True',
             u'access_level': u'Modify',
             u'free_bytes': u'59308688725',
             u'generation': u'5007',
             u'name': u'U1 - Design work',
             u'node_id': u'95276b3a-f972-4968-a4fd-30a3f68fc60a',
             u'other_username': u'ivanka',
             u'other_visible_name': u'Ivanka Majic',
             u'path': os.path.expanduser(u'~/.local/share/ubuntuone/shares/'
                                          'U1 - from Iva Majic'),
             u'subscribed': u'',
             u'type': u'Share',
             u'volume_id': u'aaae5885-d9b1-4013-a243-f570d545c124'},
        ]
        folders = [
            {u'generation': u'9',
             u'node_id': u'ec18-4943-ac22-34304391f7f4',
             u'path': os.path.expanduser(u'~/.ubuntuone/Purchased from '
                                         'Ubuntu One'),
             u'subscribed': u'True',
             u'suggested_path': u'~/.ubuntuone/Purchased from Ubuntu One',
             u'type': u'UDF',
             u'volume_id': u'f5f1-454f-8184-9aaf843a83e1'},
            {u'generation': u'4563',
             u'node_id': u'1ca3-4bc7-86ec-36c516c43772',
             u'path': os.path.expanduser(u'~/Documents'),
             u'subscribed': u'True',
             u'suggested_path': u'~/Documents',
             u'type': u'UDF',
             u'volume_id': u'e63c-471e-ac3e-c63b2d45f977'},
            {u'generation': u'12',
             u'node_id': u'a7c7-4d76-a5e3-5df506221b48',
             u'path': os.path.expanduser(u'~/testudf'),
             u'subscribed': u'',
             u'suggested_path': u'~/testudf',
             u'type': u'UDF',
             u'volume_id': u'3d28-46ae-9999-d5f48de9f460'},
        ]

        def get_shares(self):
            """Get the list of shares (accepted or not)."""
            return self.shares

        def get_folders(self):
            """Return the list of folders (a list of dicts)."""
            return self.folders

        def quit(self):
            """Quit the syncdaemon."""
            subprocess.Popen(('u1sdtool', '-q'))

        def connect(self):
            """Connect syncdaemon."""
            subprocess.Popen(('u1sdtool', '-c'))

        def disconnect(self):
            """Disconnect syncdaemon."""
            subprocess.Popen(('u1sdtool', '-d'))

        def start(self):
            """Start syncdaemon if it's not running."""
            subprocess.Popen(('u1sdtool', '-s'))

        def subscribe_share(self, share_id):
            """Subscribe to a user defined share given its id."""
            subprocess.Popen(('u1sdtool', '--subscribe-share', share_id))

        def unsubscribe_share(self, share_id):
            """Unsubscribe from a user defined share given its id."""
            subprocess.Popen(('u1sdtool', '--unsubscribe-share', share_id))

        def subscribe_folder(self, folder_id):
            """Subscribe to a user defined folder given its id."""
            subprocess.Popen(('u1sdtool', '--subscribe-folder', folder_id))

        def unsubscribe_folder(self, folder_id):
            """Unsubscribe from a user defined folder given its id."""
            subprocess.Popen(('u1sdtool', '--unsubscribe-folder', folder_id))

    from ubuntuone.platform.linux import tools
    tools.SyncDaemonTool = SDTool

    # pylint believes that reactor has no run nor stop methods. Silence it.
    # pylint: disable=E1101
    window = MainWindow(close_callback=reactor.stop)
    window.show()

    reactor.run()
