# -*- coding: utf-8 -*-

# Authors: Alejandro J. Cura <alecu@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""The test suite for the Qt UI for the control panel for Ubuntu One."""

from ubuntuone.controlpanel import backend
from ubuntuone.controlpanel.tests import TestCase
from ubuntuone.controlpanel.gui.tests import FakedObject

# Attribute 'yyy' defined outside __init__, access to a protected member
# pylint: disable=W0201, W0212


SAMPLE_NAME = "sample name"
SAMPLE_EMAIL = "sample@email"
SAMPLE_PLAN = "sample storage plan"
SAMPLE_ACCOUNT_INFO = {
    "name": SAMPLE_NAME,
    "email": SAMPLE_EMAIL,
    "type": SAMPLE_PLAN,
}
NO_OP = lambda *args: None


def skip_if_abstract_class(test):
    """Decorator to skip a test if is an abstract class."""

    def inner(instance):
        """Skip a test if is an abstract class."""
        abstract = any(i is None for i in (instance.innerclass_ui,
                                           instance.innerclass_name,
                                           instance.class_ui))
        result = None
        if not abstract:
            result = test(instance)

        return result

    return inner


class FakeUi(FakedObject):
    """A fake Ui object."""

    exposed_methods = ['setupUi']


class FakedControlPanelBackend(FakedObject):
    """Fake a Control Panel Backend."""

    ROOT_TYPE = u'ROOT'
    FOLDER_TYPE = u'UDF'
    SHARE_TYPE = u'SHARE'
    NAME_NOT_SET = u'ENAMENOTSET'
    FREE_BYTES_NOT_AVAILABLE = u'EFREEBYTESNOTAVAILABLE'

    exposed_methods = [
        'account_info',  # account
        'devices_info', 'change_device_settings', 'remove_device',  # devices
        'volumes_info', 'change_volume_settings',  # volumes
        'replications_info', 'change_replication_settings',  # replications
        'file_sync_status', 'enable_files', 'disable_files',  # files
        'connect_files', 'disconnect_files',
        'restart_files', 'start_files', 'stop_files', 'shutdown',
    ]


class FakedConfirmDialog(object):
    """Fake a confirmation dialog."""

    response = args = kwargs = None

    @classmethod
    def warning(cls, *a, **kw):
        """Simulate a warning message."""
        cls.args = a
        cls.kwargs = kw
        return cls.response


class BaseTestCase(TestCase):
    """Base Test Case."""

    innerclass_ui = None
    innerclass_name = None
    class_ui = None
    kwargs = {}

    @skip_if_abstract_class
    def setUp(self):
        super(BaseTestCase, self).setUp()
        self.patch(backend, 'ControlBackend', FakedControlPanelBackend)
        # self.class_ui is not callable
        # pylint: disable=E1102
        self.ui = self.class_ui(**self.kwargs)

        if hasattr(self.ui, 'backend'):
            # clean backend calls
            self.ui.backend._called.clear()

        self.ui.show()

    def assert_backend_called(self, method_name, args=None, kwargs=None):
        """Check that the control panel backend 'method_name' was called."""
        if args is None:
            args = ()
        if kwargs is None:
            kwargs = {}

        self.assertIn(method_name, self.ui.backend._called)
        self.assertEqual(self.ui.backend._called[method_name], (args, kwargs))

    @skip_if_abstract_class
    def test_init_loads_ui(self):
        """The __init__ method loads the ui."""
        self.patch(self.innerclass_ui, self.innerclass_name, FakeUi)
        # pylint: disable=E1102
        self.ui = self.class_ui(**self.kwargs)
        # pylint: disable=E1101
        self.assertEqual(self.ui.ui._called,
                         {'setupUi': ((self.ui,), {})})
