# -*- coding: utf-8 -*-

# Authors: Natalia B Bidart <natalia.bidart@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the Control Panel."""

import logging
import operator

from twisted.internet import defer
from ubuntuone.devtools.handlers import MementoHandler

from ubuntuone.controlpanel.gui.qt import folders as gui
from ubuntuone.controlpanel.gui.qt.tests import (
    BaseTestCase, FakedConfirmDialog,
)
from ubuntuone.controlpanel.gui.tests import (
    FAKE_VOLUMES_INFO,
    FAKE_VOLUMES_MINIMAL_INFO,
    FAKE_VOLUMES_NO_FREE_SPACE_INFO,
    MUSIC_FOLDER, ROOT, USER_HOME,
)


# Access to a protected member
# pylint: disable=W0212


class FoldersPanelTestCase(BaseTestCase):
    """Test the qt cloud folders tab."""

    innerclass_ui = gui.folders_ui
    innerclass_name = "Ui_Form"
    class_ui = gui.FoldersPanel

    def setUp(self):
        super(FoldersPanelTestCase, self).setUp()
        self.memento = MementoHandler()
        self.memento.setLevel(logging.DEBUG)
        gui.logger.addHandler(self.memento)

    def test_backend(self):
        """Backend is created."""
        self.assertIsInstance(self.ui.backend,
                              gui.backend.ControlBackend)

    def test_is_processing_before_asking_volume_info(self):
        """The ui is processing when contents are load."""
        def check():
            """The ui must be is_processing."""
            # the ui is processing
            self.assertTrue(self.ui.is_processing, 'ui must be processing')

        self.patch(self.ui.backend, 'volumes_info', check)

        self.ui.show()  # trigger the volumes_info request

    def test_is_not_processing_after_info_ready(self):
        """The ui is not processing when contents are load."""
        self.ui.process_folders(FAKE_VOLUMES_INFO)

        self.assertFalse(self.ui.is_processing)

    def test_volumes_info_is_requested_on_show(self):
        """The volumes info is requested to the backend."""
        self.assert_backend_called('volumes_info', ())

    def test_process_folders(self):
        """The volumes info is processed when ready."""
        self.ui.process_folders(FAKE_VOLUMES_INFO)
        folders = self.ui.ui.folders

        root = folders.invisibleRootItem()
        self.assertEqual(len(FAKE_VOLUMES_INFO) + 1, root.childCount())

        treeiter = gui.QtGui.QTreeWidgetItemIterator(folders)
        for name, free_bytes, volumes in FAKE_VOLUMES_INFO:
            name = "%s's" % name if name else gui.MY_FOLDERS
            free_bytes = self.ui.humanize(int(free_bytes))
            header = (name, gui.FREE_SPACE % {'free_space': free_bytes})

            # get first child, matching "My Folders"
            item = treeiter.value()
            self.assertFalse(item.is_empty)

            model_index = folders.indexFromItem(item, gui.FOLDER_NAME_COL)
            label = folders.indexWidget(model_index).text()
            self.assertEqual(label, gui.ROW_HEADER % header)

            icon_name = item.icon(gui.FOLDER_NAME_COL).name()
            self.assertEqual(icon_name, gui.CONTACT_ICON_NAME)

            # check children
            self.assertEqual(len(volumes), item.childCount())
            sorted_vols = sorted(volumes, key=operator.itemgetter('path'))
            for volume in sorted_vols:
                treeiter += 1
                item = treeiter.value()  # get child folder
                self.assertTrue(item is not None)

                name = volume['path'].replace(USER_HOME + '/', '')
                if volume['type'] == self.ui.backend.ROOT_TYPE:
                    name = gui.ROOT % (name, gui.ORANGE, gui.ALWAYS_SUBSCRIBED)
                elif volume['type'] == self.ui.backend.SHARE_TYPE:
                    name = volume['name']
                model_index = folders.indexFromItem(item, gui.FOLDER_NAME_COL)
                label = folders.indexWidget(model_index).text()
                self.assertEqual(label, name)

                subscribed = item.checkState(gui.SUBSCRIPTION_COL) == \
                             gui.CHECKED
                self.assertEqual(subscribed, bool(volume['subscribed']))

                icon_name = item.icon_obj.icon_name
                if volume['type'] != self.ui.backend.SHARE_TYPE:
                    self.assertEqual(icon_name, gui.FOLDER_ICON_NAME)
                else:
                    self.assertEqual(icon_name, gui.SHARE_ICON_NAME)

                flags = gui.QtCore.Qt.ItemIsSelectable | \
                        gui.QtCore.Qt.ItemIsUserCheckable | \
                        gui.QtCore.Qt.ItemIsEnabled | \
                        gui.QtCore.Qt.ItemIsDragEnabled | \
                        gui.QtCore.Qt.ItemIsDropEnabled
                if volume['type'] == self.ui.backend.ROOT_TYPE:
                    flags = gui.QtCore.Qt.ItemIsSelectable | \
                            gui.QtCore.Qt.ItemIsEnabled
                self.assertEqual(item.flags(), flags)

                self.assertEqual(item.volume_id, volume['volume_id'])
                self.assertEqual(item.volume_path, volume['path'])

            treeiter += 1
            item = treeiter.value()

            if item is not None:
                self.assertTrue(item.is_empty)
                # skip the empty row
                treeiter += 1

    def test_process_folders_clears_the_list(self):
        """The old volumes info is cleared before updated."""
        self.ui.process_folders(FAKE_VOLUMES_INFO)
        self.ui.process_folders(FAKE_VOLUMES_INFO)

        root = self.ui.ui.folders.invisibleRootItem()
        self.assertEqual(len(FAKE_VOLUMES_INFO) + 1, root.childCount())

    def test_process_folders_with_no_volumes(self):
        """When there are no volumes, a notification is shown."""
        self.ui.process_folders([])

        root = self.ui.ui.folders.invisibleRootItem()
        self.assertEqual(root.childCount(), 0)

    def test_process_folders_highlights_little_free_space(self):
        """The free space is red if is zero (or close to 0)."""
        self.ui.process_folders(FAKE_VOLUMES_NO_FREE_SPACE_INFO)
        folders = self.ui.ui.folders

        child_index = 0
        root = self.ui.ui.folders.invisibleRootItem()
        for name, free_bytes, _ in FAKE_VOLUMES_NO_FREE_SPACE_INFO:
            name = "%s's" % name if name else gui.MY_FOLDERS
            free_bytes = self.ui.humanize(int(free_bytes))
            free_bytes = gui.NO_FREE_SPACE % {'free_space': free_bytes}

            item = root.child(child_index)
            self.assertFalse(item.is_empty)

            model_index = folders.indexFromItem(item, gui.FOLDER_NAME_COL)
            label = folders.indexWidget(model_index).text()
            self.assertEqual(label, gui.ROW_HEADER % (name, free_bytes))

            child_index += 1
            item = root.child(child_index)

            if item is not None:
                # skip the empty row
                self.assertTrue(item.is_empty)
                child_index += 1

    def test_process_folders_handles_no_quota_info(self):
        """The lack of free space is handled."""
        info = [
            (u'', self.ui.backend.FREE_BYTES_NOT_AVAILABLE, [ROOT]),
            (u'No free space available',
             self.ui.backend.FREE_BYTES_NOT_AVAILABLE,
             [{u'volume_id': u'0', u'name': u'full', u'display_name': u'test',
               u'path': u'full-share', u'subscribed': u'',
               u'type': self.ui.backend.SHARE_TYPE}]),
        ]
        self.ui.process_folders(info)
        folders = self.ui.ui.folders

        child_index = 0
        root = self.ui.ui.folders.invisibleRootItem()
        for name, _, _ in info:
            name = "%s's" % name if name else gui.MY_FOLDERS

            item = root.child(child_index)
            self.assertFalse(item.is_empty)

            model_index = folders.indexFromItem(item, gui.FOLDER_NAME_COL)
            label = folders.indexWidget(model_index).text()
            self.assertEqual(label, gui.ROW_HEADER % (name, ''))

            child_index += 1
            item = root.child(child_index)

            if item is not None:
                # skip the empty row
                self.assertTrue(item.is_empty)
                child_index += 1

    def test_clicking_on_row_opens_folder(self):
        """The folder activated is opened."""
        self.patch(gui.os.path, 'exists', lambda *a: True)
        self.patch(gui, 'uri_hook', self._set_called)

        path = ROOT['path']
        item = gui.QtGui.QTreeWidgetItem()
        item.volume_path = path
        self.ui.on_folders_itemActivated(item)

        self.assertEqual(self._called,
                         ((gui.FILE_URI_PREFIX + path,), {}))

    def test_clicking_on_row_handles_path_none(self):
        """None paths are properly handled."""
        self.patch(gui, 'uri_hook', self._set_called)

        item = gui.QtGui.QTreeWidgetItem()
        item.volume_path = None
        self.ui.on_folders_itemActivated(item)

        self.assertTrue(self.memento.check_warning(str(item),
                                                   'volume_path', 'is None'))
        self.assertEqual(self._called, False)

    def test_clicking_on_row_handles_path_non_existent(self):
        """Not-existent paths are properly handled."""
        self.patch(gui.os.path, 'exists', lambda *a: False)
        self.patch(gui, 'uri_hook', self._set_called)

        path = 'not-in-disk'
        item = gui.QtGui.QTreeWidgetItem()
        item.volume_path = path
        self.ui.on_folders_itemActivated(item)

        self.assertTrue(self.memento.check_warning(path, 'does not exist'))
        self.assertEqual(self._called, False)

    def test_process_folders_with_music_folder(self):
        """The volumes info is processed when ready."""
        self.ui.process_folders(FAKE_VOLUMES_MINIMAL_INFO)
        folders = self.ui.ui.folders

        treeiter = gui.QtGui.QTreeWidgetItemIterator(folders)

        # walk to 'My folders' children
        treeiter += 1

        # grab next row since first one is root
        treeiter += 1
        item = treeiter.value()

        volume = MUSIC_FOLDER

        model_index = folders.indexFromItem(item, gui.FOLDER_NAME_COL)
        label = folders.indexWidget(model_index).text()
        self.assertEqual(label, gui.MUSIC_DISPLAY_NAME)

        subscribed = item.checkState(gui.SUBSCRIPTION_COL) == gui.CHECKED
        self.assertEqual(subscribed, bool(volume['subscribed']))

        icon_name = item.icon_obj.icon_name
        self.assertEqual(icon_name, gui.MUSIC_ICON_NAME)

        self.assertEqual(item.volume_id, volume['volume_id'])
        self.assertEqual(item.volume_path, volume['path'])


class FoldersPanelSubscriptionTestCase(FoldersPanelTestCase):
    """The test suite for the folder subscription."""

    def setUp(self):
        super(FoldersPanelSubscriptionTestCase, self).setUp()
        self.patch(gui.os.path, 'exists', lambda path: True)
        FakedConfirmDialog.response = gui.YES
        FakedConfirmDialog.args = None
        FakedConfirmDialog.kwargs = None
        self.patch(gui.QtGui, 'QMessageBox', FakedConfirmDialog)

        self.ui.process_folders(FAKE_VOLUMES_MINIMAL_INFO)
        # the music folder
        self.item = self.ui.ui.folders.topLevelItem(0).child(1)

    @defer.inlineCallbacks
    def test_on_folders_item_changed(self):
        """Clicking on 'subscribed' updates the folder subscription."""
        volume = MUSIC_FOLDER
        fid = volume['volume_id']
        subscribed = not bool(volume['subscribed'])
        check_state = gui.CHECKED if subscribed else gui.UNCHECKED

        self.ui.is_processing = True
        self.item.setCheckState(gui.SUBSCRIPTION_COL, check_state)
        self.ui.is_processing = False

        yield self.ui.on_folders_itemChanged(self.item)

        # backend was called
        self.assert_backend_called('change_volume_settings',
                                   (fid, {'subscribed': subscribed}))

        value = self.item.checkState(gui.SUBSCRIPTION_COL) == gui.CHECKED
        self.assertEqual(value, bool(subscribed))

    @defer.inlineCallbacks
    def test_on_folders_item_changed_is_processing(self):
        """Clicking on 'subscribed' sets is_processing flag until done."""
        def check(volume_id, settings):
            """The ui must be is_processing when a change was requested."""
            # the ui is processing
            self.assertTrue(self.ui.is_processing, 'ui must be processing')

        self.patch(self.ui.backend, 'change_volume_settings', check)

        yield self.ui.on_folders_itemChanged(self.item)

        # the ui is no loner processing
        self.assertFalse(self.ui.is_processing, 'ui must not be processing')

    @defer.inlineCallbacks
    def test_on_folders_item_changed_volume_id_is_none(self):
        """If item.volume_id is None, log warning and return."""
        self.patch(self.ui.backend, 'change_volume_settings', self._set_called)

        self.item.volume_id = None
        yield self.ui.on_folders_itemChanged(self.item)

        self.assertTrue(self.memento.check_warning(str(self.item),
                                                   'volume_id', 'is None'))
        self.assertEqual(self._called, False)

    @defer.inlineCallbacks
    def test_on_folders_item_changed_does_nothing_if_processing(self):
        """If UI is already processing, do nothing and return."""
        self.patch(self.ui.backend, 'change_volume_settings', self._set_called)

        self.ui.is_processing = True

        yield self.ui.on_folders_itemChanged(self.item)

        self.assertEqual(self._called, False)

    @defer.inlineCallbacks
    def test_confirm_dialog_if_path_exists(self):
        """The confirmation dialog is correct."""
        self.patch(gui.os.path, 'exists', lambda path: True)

        # make sure the item is subscribed
        self.ui.is_processing = True
        self.item.setCheckState(gui.SUBSCRIPTION_COL, gui.CHECKED)
        self.ui.is_processing = False

        yield self.ui.on_folders_itemChanged(self.item)

        volume_path = self.item.volume_path
        msg = gui.FOLDERS_CONFIRM_MERGE % {'folder_path': volume_path}
        buttons = gui.YES | gui.NO | gui.CANCEL
        self.assertEqual(FakedConfirmDialog.args,
                         (self.ui, '', msg, buttons, gui.YES))
        self.assertEqual(FakedConfirmDialog.kwargs, {})

    @defer.inlineCallbacks
    def test_confirm_dialog_if_path_does_not_exist(self):
        """The confirmation dialog is not created if not needed."""
        self.patch(gui.os.path, 'exists', lambda path: False)

        # make sure the item is unsubscribed
        self.ui.is_processing = True
        self.item.setCheckState(gui.SUBSCRIPTION_COL, gui.UNCHECKED)
        self.ui.is_processing = False

        yield self.ui.on_folders_itemChanged(self.item)

        self.assertEqual(FakedConfirmDialog.args, None)
        self.assertEqual(FakedConfirmDialog.kwargs, None)

    @defer.inlineCallbacks
    def test_subscribe_does_not_call_backend_if_dialog_closed(self):
        """Backend is not called if users closes the confirmation dialog."""
        FakedConfirmDialog.response = gui.CANCEL
        self.patch(gui.QtGui, 'QMessageBox', FakedConfirmDialog)

        # make sure the item is subscribed
        self.ui.is_processing = True
        self.item.setCheckState(gui.SUBSCRIPTION_COL, gui.CHECKED)
        self.ui.is_processing = False

        yield self.ui.on_folders_itemChanged(self.item)

        self.assertFalse(FakedConfirmDialog.args is None, 'warning was called')
        self.assertNotIn('change_volume_settings', self.ui.backend._called)
        self.assertFalse(self.ui.is_processing)

        subscribed = self.item.checkState(gui.SUBSCRIPTION_COL) == gui.CHECKED
        self.assertEqual(subscribed, False)

    @defer.inlineCallbacks
    def test_subscribe_does_not_call_backend_if_answer_is_no(self):
        """Backend is not called if users clicks on 'No'."""
        FakedConfirmDialog.response = gui.NO
        self.patch(gui.QtGui, 'QMessageBox', FakedConfirmDialog)

        # make sure the item is subscribed
        self.ui.is_processing = True
        self.item.setCheckState(gui.SUBSCRIPTION_COL, gui.CHECKED)
        self.ui.is_processing = False

        yield self.ui.on_folders_itemChanged(self.item)

        self.assertFalse(FakedConfirmDialog.args is None, 'warning was called')
        self.assertNotIn('change_volume_settings', self.ui.backend._called)
        self.assertFalse(self.ui.is_processing)

        subscribed = self.item.checkState(gui.SUBSCRIPTION_COL) == gui.CHECKED
        self.assertEqual(subscribed, False)

    @defer.inlineCallbacks
    def test_no_confirmation_if_unsubscribing(self):
        """The confirmation dialog is not shown if unsubscribing."""
        # make sure the item is unsubscribed
        self.ui.is_processing = True
        self.item.setCheckState(gui.SUBSCRIPTION_COL, gui.UNCHECKED)
        self.ui.is_processing = False

        # the confirm dialog was not called so far
        assert FakedConfirmDialog.args is None
        assert FakedConfirmDialog.kwargs is None

        yield self.ui.on_folders_itemChanged(self.item)

        self.assertTrue(FakedConfirmDialog.args is None, 'dialog was not run')
        self.assertTrue(FakedConfirmDialog.kwargs is None, 'dialog was hid')
