# -*- coding: utf-8 -*-

# Authors: Alejandro J. Cura <alecu@canonical.com>
# Authors: Natalia B. Bidart <natalia.bidart@canonical.com>
#
# Copyright 2010 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the DBus service when accessing SyncDaemon."""

import uuid
import os

import dbus

from twisted.internet import defer
from twisted.internet.defer import inlineCallbacks, returnValue
from ubuntuone.platform.linux import dbus_interface as sd_dbus_iface, tools

from ubuntuone.controlpanel import sd_client
from ubuntuone.controlpanel.integrationtests import DBusClientTestCase


SAMPLE_LIMITS = {'upload': 999, 'download': 838}
SD_DBUS_IFACE_NAME = sd_dbus_iface.DBUS_IFACE_NAME
SD_DBUS_IFACE_STATUS_NAME = sd_dbus_iface.DBUS_IFACE_STATUS_NAME

# pylint, you have to go to decorator's school
# pylint: disable=C0322

# Access to a protected member of a client class
# pylint: disable=W0212


class CustomError(Exception):
    """Custom error for tests."""


class FakedSyncDaemonTool(object):
    """Fake the SyncDaemonTool."""

    settings = {
        'show_all_notifications': True,
        'file_sync_enabled': True,
        'throttling_enabled': False,
        'limits': {'download': -1, 'upload': -1},
    }
    status_dict = {
        'name': 'QUEUE_MANAGER',
        'connection': 'With User With Network',
        'description': 'processing the commands pool',
        'is_connected': True,
        'is_error': False,
        'is_online': True,
        'queues': 'IDLE',
    }
    called = {}
    shares = {}
    folders = {}
    root_dir = os.path.expanduser('~/Ubuntu One')
    shares_dir = os.path.expanduser('~/.cache/ubuntuone/shares')
    shares_dir_link = os.path.join(root_dir, 'Shared With Me')

    def __init__(self, bus):
        self.bus = bus

    @classmethod
    def create_share(cls, name, accepted=False, subscribed=False):
        """Add a new share (fake)."""
        sid = unicode(uuid.uuid4())
        path = u'/home/tester/.hidden/shares/%s from The Othr User' % name
        share = {
            u'name': name, u'subscribed': sd_dbus_iface.bool_str(subscribed),
            u'accepted': sd_dbus_iface.bool_str(accepted),
            u'generation': u'36', u'type': u'Share',
            u'node_id': u'ca3a1cec-09d2-485e-9685-1a5180bd6441',
            u'volume_id': sid, u'access_level': u'View',
            u'other_username': u'https://login.ubuntu.com/+id/nHRnYmz',
            u'other_visible_name': u'The Other User',
            u'free_bytes': u'2146703403', u'path': path,
        }
        cls.shares[sid] = share
        return share

    @classmethod
    def create_udf(cls, path, subscribed=False):
        """Create a new faked folder."""
        fid = unicode(uuid.uuid4())
        folder = {}
        if isinstance(path, str):
            path = path.decode('utf-8')
        folder[u'path'] = folder[u'suggested_path'] = path
        folder[u'volume_id'] = fid
        folder[u'subscribed'] = sd_dbus_iface.bool_str(subscribed)
        folder[u'node_id'] = u'a18f4cbd-a846-4405-aaa1-b28904817089'
        folder[u'generation'] = u''
        folder[u'type'] = u'UDF'
        cls.folders[fid] = folder
        return folder

    @classmethod
    def _set_share_attr(cls, share_id, attr, value):
        """Set values to shares."""
        if share_id not in cls.shares:
            raise CustomError('share_id %r not in shares %r.' %
                              (share_id, cls.shares))

        value = sd_dbus_iface.bool_str(value)
        cls.shares[share_id][attr] = value
        return share_id

    @classmethod
    def _set_folder_attr(cls, folder_id, attr, value):
        """Set values to folders."""
        if folder_id not in cls.folders:
            raise CustomError('folder_id %r not in folders %r.' %
                              (folder_id, cls.folders))

        value = sd_dbus_iface.bool_str(value)
        cls.folders[folder_id][attr] = value
        return folder_id

    @inlineCallbacks
    def accept_share(self, share_id):
        """Accept the share with id: share_id."""
        yield self._set_share_attr(share_id, u'accepted', True)

    @inlineCallbacks
    def reject_share(self, share_id):
        """Reject the share with id: share_id."""
        yield self._set_share_attr(share_id, u'accepted', False)

    @inlineCallbacks
    def subscribe_share(self, share_id):
        """Subscribe to a share given its id."""
        yield self._set_share_attr(share_id, u'subscribed', True)

    @inlineCallbacks
    def unsubscribe_share(self, share_id):
        """Unsubscribe from a share given its id."""
        yield self._set_share_attr(share_id, u'subscribed', False)

    @inlineCallbacks
    def get_shares(self):
        """Get the list of shares (accepted or not)."""
        result = yield FakedSyncDaemonTool.shares.values()
        returnValue(sorted(result))

    def refresh_shares(self):
        """Call refresh_shares method via DBus."""
        self.called['refresh_shares'] = None

    def offer_share(self, path, username, name, access_level):
        """Offer a share at the specified path to user with id: username."""
        self.called['offer_share'] = (path, username, name, access_level)

    def list_shared(self):
        """Get the list of the shares "shared"/created/offered."""
        self.called['list_shared'] = None

    def create_folder(self, path):
        """Create a user defined folder in the specified path."""
        if path == '':  # simulate an error
            raise CustomError(path)

        folder = FakedSyncDaemonTool.create_udf(path)
        return folder

    def delete_folder(self, folder_id):
        """Delete a user defined folder given its id."""
        if folder_id not in FakedSyncDaemonTool.folders:
            raise CustomError(folder_id)

        folder = FakedSyncDaemonTool.folders.pop(folder_id)
        return folder

    @inlineCallbacks
    def subscribe_folder(self, folder_id):
        """Subscribe to a user defined folder given its id."""
        yield FakedSyncDaemonTool._set_folder_attr(folder_id,
                                                   u'subscribed', True)

    @inlineCallbacks
    def unsubscribe_folder(self, folder_id):
        """Unsubscribe from a user defined folder given its id."""
        yield FakedSyncDaemonTool._set_folder_attr(folder_id,
                                                   u'subscribed', False)

    @inlineCallbacks
    def get_folders(self):
        """Return the list of folders (a list of dicts)."""
        result = yield FakedSyncDaemonTool.folders.values()
        returnValue(sorted(result))

    def get_folder_info(self, path):
        """Call the get_info method for a UDF path."""
        for _, content in FakedSyncDaemonTool.folders.iteritems():
            if content['path'] == path:
                return content
        else:
            return {}

    def get_metadata(self, path):
        """Call FileSystem.get_metadata."""
        self.called['get_metadata'] = path

    def change_public_access(self, path, is_public):
        """Change the public access for a given path."""
        self.called['change_public_access'] = (path, is_public)

    def quit(self):
        """Quit the syncdaemon."""
        self.called['quit'] = None

    def connect(self):
        """Connect syncdaemon."""
        self.called['connect'] = None

    def disconnect(self):
        """Disconnect syncdaemon."""
        self.called['disconnect'] = None

    def get_status(self):
        """Get the current_status dict."""
        return self.status_dict

    def start(self):
        """Start syncdaemon if it's not running."""
        self.called['start'] = None

    def get_throttling_limits(self):
        """Return a dict with the read and write limits."""
        return self.settings['limits']

    def set_throttling_limits(self, read_limit, write_limit):
        """Set the read and write limits."""
        self.settings['limits']["download"] = read_limit
        self.settings['limits']["upload"] = write_limit

    def is_throttling_enabled(self):
        """Check if throttling is enabled."""
        return self.settings['throttling_enabled']

    def enable_throttling(self, enabled):
        """Enable/disable throttling."""
        self.settings['throttling_enabled'] = enabled

    def is_files_sync_enabled(self):
        """Check if files sync is enabled."""
        return self.settings['file_sync_enabled']

    def enable_files_sync(self, enabled):
        """Enable/disable file sync."""
        self.settings['file_sync_enabled'] = enabled

    def is_show_all_notifications_enabled(self):
        """Check if show_all_notifications is enabled."""
        return self.settings['show_all_notifications']

    def enable_show_all_notifications(self, enabled):
        """Enable/disable show_all_notifications."""
        self.settings['show_all_notifications'] = enabled

    def refresh_volumes(self):
        """Request the volumes list to the server."""
        self.called['refresh_volumes'] = None

    def rescan_from_scratch(self, volume_id):
        """Request a rescan from scratch for volume_id."""
        self.called['rescan_from_scratch'] = volume_id

    def get_root_dir(self):
        """Return the root directory."""
        return self.root_dir

    def get_shares_dir(self):
        """Return the shares directory."""
        return self.shares_dir

    def get_shares_dir_link(self):
        """Return the shares link directory."""
        return self.shares_dir_link


class BaseTestCase(DBusClientTestCase):
    """Test for the syncdaemon client methods."""

    def setUp(self):
        super(BaseTestCase, self).setUp()
        self.patch(tools, 'SyncDaemonTool', FakedSyncDaemonTool)
        FakedSyncDaemonTool.shares.clear()
        FakedSyncDaemonTool.folders.clear()
        FakedSyncDaemonTool.called.clear()

    def fake_fail(self, *a):
        """Fake a failure."""
        return defer.fail(CustomError(a))


class ThrottlingTestCase(BaseTestCase):
    """Test for the throttling syncdaemon client methods."""

    @inlineCallbacks
    def test_throttle_limits_returned(self):
        """When SD returns the limits, they are handled."""
        limits = yield sd_client.get_throttling_limits()
        self.assertEqual(limits, FakedSyncDaemonTool.settings['limits'])

    @inlineCallbacks
    def test_getting_throttle_limits_throws_an_error(self):
        """Handle error when getting the limits."""
        self.patch(FakedSyncDaemonTool, 'get_throttling_limits',
                   self.fake_fail)
        yield self.assertFailure(sd_client.get_throttling_limits(),
                                 CustomError)

    @inlineCallbacks
    def test_throttle_limits_set(self):
        """Setting the limits."""
        yield sd_client.set_throttling_limits(SAMPLE_LIMITS)
        actual = sd_client.get_throttling_limits()
        self.assertEqual(actual, SAMPLE_LIMITS)

    @inlineCallbacks
    def test_setting_throttle_limits_throws_an_error(self):
        """Handle error when setting the limits."""
        self.patch(FakedSyncDaemonTool, 'set_throttling_limits',
                   self.fake_fail)
        d = sd_client.set_throttling_limits(SAMPLE_LIMITS)
        yield self.assertFailure(d, CustomError)

    @inlineCallbacks
    def test_throttle_get(self):
        """Getting the throttling state."""
        FakedSyncDaemonTool.settings['throttling_enabled'] = False
        throttling = yield sd_client.bandwidth_throttling_enabled()
        self.assertEqual(throttling, False)

        FakedSyncDaemonTool.settings['throttling_enabled'] = True
        throttling = yield sd_client.bandwidth_throttling_enabled()
        self.assertEqual(throttling, True)

    @inlineCallbacks
    def test_throttle_get_throws_an_error(self):
        """Handle error when getting the throttling state."""
        self.patch(FakedSyncDaemonTool, 'is_throttling_enabled',
                   self.fake_fail)
        yield self.assertFailure(sd_client.bandwidth_throttling_enabled(),
                                 CustomError)

    @inlineCallbacks
    def test_throttle_enable(self):
        """Enabling bw throttling."""
        FakedSyncDaemonTool.settings['throttling_enabled'] = False
        yield sd_client.enable_bandwidth_throttling()
        self.assertTrue(FakedSyncDaemonTool.settings['throttling_enabled'])

    @inlineCallbacks
    def test_throttle_enable_throws_an_error(self):
        """Handle error when enabling throttling."""
        self.patch(FakedSyncDaemonTool, 'enable_throttling',
                   self.fake_fail)
        yield self.assertFailure(sd_client.enable_bandwidth_throttling(),
                                 CustomError)

    @inlineCallbacks
    def test_throttle_disable(self):
        """Disabling bw throttling."""
        FakedSyncDaemonTool.settings['throttling_enabled'] = True
        yield sd_client.disable_bandwidth_throttling()
        self.assertFalse(FakedSyncDaemonTool.settings['throttling_enabled'])

    @inlineCallbacks
    def test_throttle_disable_throws_an_error(self):
        """Handle error when disabling throttling."""
        self.patch(FakedSyncDaemonTool, 'enable_throttling',
                   self.fake_fail)
        yield self.assertFailure(sd_client.disable_bandwidth_throttling(),
                                 CustomError)


class NotificationsTestCase(BaseTestCase):
    """Test for the show_all_notifications syncdaemon client methods."""

    @inlineCallbacks
    def test_notifications_get(self):
        """Getting the show_all_notifications state."""
        FakedSyncDaemonTool.settings['show_all_notifications'] = False
        state = yield sd_client.show_all_notifications_enabled()
        self.assertEqual(state, False)
        FakedSyncDaemonTool.settings['show_all_notifications'] = True
        state = yield sd_client.show_all_notifications_enabled()
        self.assertEqual(state, True)

    @inlineCallbacks
    def test_notifications_get_throws_an_error(self):
        """Handle error when getting the show_all_notifications state."""
        self.patch(FakedSyncDaemonTool, 'is_show_all_notifications_enabled',
                   self.fake_fail)
        yield self.assertFailure(sd_client.show_all_notifications_enabled(),
                                 CustomError)

    @inlineCallbacks
    def test_notifications_enable(self):
        """Enabling show_all_notifications."""
        FakedSyncDaemonTool.settings['show_all_notifications'] = False
        yield sd_client.enable_show_all_notifications()
        self.assertTrue(FakedSyncDaemonTool.settings['show_all_notifications'])

    @inlineCallbacks
    def test_notifications_enable_throws_an_error(self):
        """Handle error when enabling show_all_notifications."""
        self.patch(FakedSyncDaemonTool, 'enable_show_all_notifications',
                   self.fake_fail)
        yield self.assertFailure(sd_client.enable_show_all_notifications(),
                                 CustomError)

    @inlineCallbacks
    def test_notifications_disable(self):
        """Disabling show_all_notifications."""
        FakedSyncDaemonTool.settings['show_all_notifications'] = True
        yield sd_client.disable_show_all_notifications()
        value = FakedSyncDaemonTool.settings['show_all_notifications']
        self.assertFalse(value)

    @inlineCallbacks
    def test_notifications_disable_throws_an_error(self):
        """Handle error when disabling show_all_notifications."""
        self.patch(FakedSyncDaemonTool, 'enable_show_all_notifications',
                   self.fake_fail)
        yield self.assertFailure(sd_client.disable_show_all_notifications(),
                                 CustomError)


class FoldersTestCase(BaseTestCase):
    """Test for the shares syncdaemon client methods."""

    @inlineCallbacks
    def test_get_folders(self):
        """Retrieve folders info list."""
        folder1 = FakedSyncDaemonTool.create_udf(path='~/bar/baz')
        folder2 = FakedSyncDaemonTool.create_udf(path='~/bar/foo',
                                                 subscribed=False)
        result = yield sd_client.get_folders()

        self.assertEqual(result, sorted([folder1, folder2]))

    @inlineCallbacks
    def test_get_folders_error(self):
        """Handle error when retrieving current syncdaemon status."""
        self.patch(FakedSyncDaemonTool, 'get_folders', self.fake_fail)
        yield self.assertFailure(sd_client.get_folders(), CustomError)

    @inlineCallbacks
    def test_create_folder(self):
        """Create a new folder."""
        path = '~/bar/baz'
        folder_info = yield sd_client.create_folder(path)

        expected = FakedSyncDaemonTool.folders[folder_info['volume_id']]
        self.assertEqual(expected, folder_info)

    @inlineCallbacks
    def test_create_folder_error(self):
        """Create a new folder fails."""
        self.patch(FakedSyncDaemonTool, 'create_folder', self.fake_fail)
        path = ''
        failure = yield self.assertFailure(sd_client.create_folder(path=path),
                                           CustomError)
        self.assertEqual(failure[0], (path,))

    @inlineCallbacks
    def test_subscribe_folder(self):
        """Subscribe to a folder."""
        folder_info = FakedSyncDaemonTool.create_udf(path='~/bar/foo',
                                                     subscribed=False)
        fid = folder_info['volume_id']
        yield sd_client.subscribe_folder(fid)

        result = yield sd_client.get_folders()
        expected, = filter(lambda folder: folder['volume_id'] == fid, result)
        self.assertEqual(expected['subscribed'], 'True')

    @inlineCallbacks
    def test_subscribe_folder_error(self):
        """Subscribe to a folder."""
        self.patch(FakedSyncDaemonTool, 'subscribe_folder', self.fake_fail)
        fid = u'does not exist'
        failure = yield self.assertFailure(sd_client.subscribe_folder(fid),
                                           CustomError)
        self.assertEqual(failure[0], (fid,))

    @inlineCallbacks
    def test_unsubscribe_folder(self):
        """Unsubscribe to a folder."""
        folder_info = FakedSyncDaemonTool.create_udf(path='~/bar/foo',
                                                     subscribed=True)
        fid = folder_info['volume_id']

        yield sd_client.unsubscribe_folder(fid)

        self.assertEqual(FakedSyncDaemonTool.folders[fid]['subscribed'], '')

    @inlineCallbacks
    def test_unsubscribe_folder_error(self):
        """Unsubscribe to a folder."""
        self.patch(FakedSyncDaemonTool, 'unsubscribe_folder', self.fake_fail)
        fid = u'does not exist'
        failure = yield self.assertFailure(sd_client.unsubscribe_folder(fid),
                                           CustomError)
        self.assertEqual(failure[0], (fid,))


class SharesTestCase(BaseTestCase):
    """Test for the shares syncdaemon client methods."""

    @inlineCallbacks
    def test_get_shares(self):
        """Retrieve shares info list."""
        share1 = FakedSyncDaemonTool.create_share(name='first, test me!')
        share2 = FakedSyncDaemonTool.create_share(name='and test me more!',
                                                accepted=True)
        share3 = FakedSyncDaemonTool.create_share(name='last but not least',
                                                accepted=True, subscribed=True)

        result = yield sd_client.get_shares()

        self.assertEqual(result, sorted([share1, share2, share3]))

    @inlineCallbacks
    def test_get_shares_error(self):
        """Handle error when retrieving current syncdaemon status."""
        self.patch(FakedSyncDaemonTool, 'get_shares', self.fake_fail)
        yield self.assertFailure(sd_client.get_shares(), CustomError)

    @inlineCallbacks
    def test_subscribe_share(self):
        """Subscribe to a share."""
        share = FakedSyncDaemonTool.create_share(name='to be subscribed',
                                               accepted=True, subscribed=False)
        sid = share['volume_id']

        yield sd_client.subscribe_share(sid)

        result = yield sd_client.get_shares()
        expected, = filter(lambda share: share['volume_id'] == sid, result)
        self.assertEqual(expected['subscribed'], 'True')

    @inlineCallbacks
    def test_subscribe_share_error(self):
        """Subscribe to a share."""
        sid = u'does not exist'
        yield self.assertFailure(sd_client.subscribe_share(sid), CustomError)

    @inlineCallbacks
    def test_unsubscribe_share(self):
        """Unsubscribe to a share."""
        share = FakedSyncDaemonTool.create_share(name='to be unsubscribed',
                                                accepted=True, subscribed=True)
        sid = share['volume_id']

        yield sd_client.unsubscribe_share(sid)

        result = yield sd_client.get_shares()
        expected, = filter(lambda share: share['volume_id'] == sid, result)
        self.assertEqual(expected['subscribed'], '')

    @inlineCallbacks
    def test_unsubscribe_share_error(self):
        """Unsubscribe to a share."""
        sid = u'does not exist'
        yield self.assertFailure(sd_client.unsubscribe_share(sid), CustomError)


class StatusMockDBusSyncDaemon(dbus.service.Object):
    """A mock object that mimicks syncdaemon regarding the Status iface."""

    state_dict = {
        'name': 'TEST',
        'description': 'Some test state, nothing else.',
        'is_error': '',
        'is_connected': 'True',
        'is_online': '',
        'queues': 'GORGEOUS',
        'connection': '',
    }

    def _get_current_state(self):
        """Get the current status of the system."""
        return self.state_dict

    @dbus.service.method(SD_DBUS_IFACE_STATUS_NAME,
                         in_signature='', out_signature='a{ss}')
    def current_status(self):
        """Return the current faked status of the system."""
        return self._get_current_state()

    # pylint: disable=C0103
    # Invalid name "StatusChanged"

    @dbus.service.signal(SD_DBUS_IFACE_STATUS_NAME)
    def StatusChanged(self, status):
        """Fire a signal to notify that the status of the system changed."""

    def emit_status_changed(self, state=None):
        """Emit StatusChanged."""
        self.StatusChanged(self._get_current_state())


class StatusTestCase(DBusClientTestCase):
    """Test for the status dbus client methods."""

    def setUp(self):
        super(StatusTestCase, self).setUp()
        self.register_mockserver(SD_DBUS_IFACE_NAME,
                                 "/status", StatusMockDBusSyncDaemon)

    def test_set_status_changed_handler(self):
        """A proper callback can be connected to StatusChanged signal."""
        _, sig = sd_client.set_status_changed_handler(self._set_called)

        self.assertEqual(sig._handler, self._set_called)
        self.assertEqual(sig._member, 'StatusChanged')
        self.assertEqual(sig._path, '/status')
        self.assertEqual(sig._interface, SD_DBUS_IFACE_STATUS_NAME)


class FileSyncTestCase(BaseTestCase):
    """Test for the file sync syncdaemon client methods."""

    @inlineCallbacks
    def test_file_sync_enabled(self):
        """Retrieve whether file sync is enabled."""
        expected = object()
        FakedSyncDaemonTool.settings['file_sync_enabled'] = expected

        enabled = yield sd_client.file_sync_enabled()

        self.assertEqual(expected, enabled)

    @inlineCallbacks
    def test_enable_file_sync(self):
        """Enable the file sync service."""
        yield sd_client.enable_file_sync()

        self.assertTrue(FakedSyncDaemonTool.settings['file_sync_enabled'])

    @inlineCallbacks
    def test_disable_file_sync(self):
        """Disable the file sync service."""
        yield sd_client.disable_file_sync()

        self.assertFalse(FakedSyncDaemonTool.settings['file_sync_enabled'])

    @inlineCallbacks
    def test_connect_file_sync(self):
        """Set if file sync is enabled or not."""
        yield sd_client.connect_file_sync()

        self.assertEqual(FakedSyncDaemonTool.called['connect'], None)

    @inlineCallbacks
    def test_disconnect_file_sync(self):
        """Set if file sync is enabled or not."""
        yield sd_client.disconnect_file_sync()

        self.assertEqual(FakedSyncDaemonTool.called['disconnect'], None)

    @inlineCallbacks
    def test_start_file_sync(self):
        """Set if file sync is enabled or not."""
        yield sd_client.start_file_sync()

        self.assertEqual(FakedSyncDaemonTool.called['start'], None)

    @inlineCallbacks
    def test_stop_file_sync(self):
        """Set if file sync is enabled or not."""
        yield sd_client.stop_file_sync()

        self.assertEqual(FakedSyncDaemonTool.called['quit'], None)


class BasicTestCase(BaseTestCase):
    """Test for the basic syncdaemon client methods."""

    @inlineCallbacks
    def test_get_root_dir(self):
        """Retrieve current syncdaemon root dir."""
        root = yield sd_client.get_root_dir()

        self.assertEqual(FakedSyncDaemonTool.root_dir, root)

    @inlineCallbacks
    def test_get_shares_dir(self):
        """Retrieve current syncdaemon shares dir."""
        result = yield sd_client.get_shares_dir()

        self.assertEqual(FakedSyncDaemonTool.shares_dir, result)

    @inlineCallbacks
    def test_get_shares_dir_link(self):
        """Retrieve current syncdaemon shares dir."""
        result = yield sd_client.get_shares_dir_link()

        self.assertEqual(FakedSyncDaemonTool.shares_dir_link, result)

    @inlineCallbacks
    def test_get_current_status(self):
        """Retrieve current syncdaemon status."""
        status = yield sd_client.get_current_status()

        self.assertEqual(FakedSyncDaemonTool.status_dict, status)

    @inlineCallbacks
    def test_get_current_status_error(self):
        """Handle error when retrieving current syncdaemon status."""
        self.patch(FakedSyncDaemonTool, 'get_status', self.fake_fail)
        yield self.assertFailure(sd_client.get_current_status(), CustomError)
