# -*- coding: utf-8 -*-

# Authors: Alejandro J. Cura <alecu@canonical.com>
# Authors: Natalia B. Bidart <nataliabidart@canonical.com>
#
# Copyright 2010 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Client to use other DBus services."""

import warnings

import dbus.service

from ubuntuone.controlpanel.logger import setup_logging


logger = setup_logging('sd_client')


def get_syncdaemon_tool():
    """Get a proxy for the SyncDaemonTool."""
    # Reimport 'SyncDaemonTool'
    # pylint: disable=W0404
    from ubuntuone.platform.linux.tools import SyncDaemonTool
    proxy = SyncDaemonTool(bus=dbus.SessionBus())
    return proxy


def get_syncdaemon_proxy(object_path, dbus_interface):
    """Get a DBus proxy for syncdaemon at 'object_path':'dbus_interface'."""
    logger.debug('get_syncdaemon_proxy: object_path %r, dbus_interface %r',
                 object_path, dbus_interface)
    bus = dbus.SessionBus()
    obj = bus.get_object(bus_name='com.ubuntuone.SyncDaemon',
                         object_path=object_path,
                         follow_name_owner_changes=True)
    proxy = dbus.Interface(object=obj, dbus_interface=dbus_interface)
    return proxy


def get_throttling_limits():
    """Get the speed limits from the syncdaemon."""
    return get_syncdaemon_tool().get_throttling_limits()


def set_throttling_limits(limits):
    """Set the speed limits on the syncdaemon."""
    dload = int(limits["download"])
    uload = int(limits["upload"])
    return get_syncdaemon_tool().set_throttling_limits(dload, uload)


def bandwidth_throttling_enabled():
    """Get the state of throttling in the syncdaemon."""
    return get_syncdaemon_tool().is_throttling_enabled()


def enable_bandwidth_throttling():
    """Enable the speed limits in the syncdaemon."""
    return get_syncdaemon_tool().enable_throttling(True)


def disable_bandwidth_throttling():
    """Disable the speed limits in the syncdaemon."""
    return get_syncdaemon_tool().enable_throttling(False)


def show_all_notifications_enabled():
    """Get the state of show_all_notifications in the syncdaemon."""
    return get_syncdaemon_tool().is_show_all_notifications_enabled()


def enable_show_all_notifications():
    """Enable show_all_notifications in the syncdaemon."""
    return get_syncdaemon_tool().enable_show_all_notifications(True)


def disable_show_all_notifications():
    """Disable show_all_notifications in the syncdaemon."""
    return get_syncdaemon_tool().enable_show_all_notifications(False)


def get_root_dir():
    """Retrieve the root information from syncdaemon."""
    return get_syncdaemon_tool().get_root_dir()


def get_shares_dir():
    """Retrieve the shares information from syncdaemon."""
    return get_syncdaemon_tool().get_shares_dir()


def get_shares_dir_link():
    """Retrieve the shares information from syncdaemon."""
    return get_syncdaemon_tool().get_shares_dir_link()


def get_folders():
    """Retrieve the folders information from syncdaemon."""
    return get_syncdaemon_tool().get_folders()


def create_folder(path):
    """Create a new folder through syncdaemon."""
    return get_syncdaemon_tool().create_folder(path)


def subscribe_folder(folder_id):
    """Subscribe to 'folder_id'."""
    return get_syncdaemon_tool().subscribe_folder(folder_id)


def unsubscribe_folder(folder_id):
    """Unsubscribe 'folder_id'."""
    return get_syncdaemon_tool().unsubscribe_folder(folder_id)


def get_shares():
    """Retrieve the shares information from syncdaemon."""
    return get_syncdaemon_tool().get_shares()


def subscribe_share(share_id):
    """Subscribe to 'share_id'."""
    return get_syncdaemon_tool().subscribe_share(share_id)


def unsubscribe_share(share_id):
    """Unsubscribe 'share_id'."""
    return get_syncdaemon_tool().unsubscribe_share(share_id)


def get_current_status():
    """Retrieve the current status from syncdaemon."""
    return get_syncdaemon_tool().get_status()


def set_status_changed_handler(handler):
    """Connect 'handler' with syncdaemon's StatusChanged signal."""
    proxy = get_syncdaemon_proxy('/status', 'com.ubuntuone.SyncDaemon.Status')
    sig = proxy.connect_to_signal('StatusChanged', handler)
    return proxy, sig


def file_sync_enabled():
    """Get if file sync service is enabled."""
    return get_syncdaemon_tool().is_files_sync_enabled()


def enable_file_sync():
    """Enable the file sync service."""
    return get_syncdaemon_tool().enable_files_sync(True)


def disable_file_sync():
    """Enable the file sync service."""
    return get_syncdaemon_tool().enable_files_sync(False)


def files_sync_enabled():
    """Get if file sync service is enabled."""
    warnings.warn('use file_sync_enabled instead', DeprecationWarning)
    return file_sync_enabled()


def set_files_sync_enabled(enabled):
    """Set the file sync service to be 'enabled'."""
    warnings.warn('use {enable/disable}_file_sync instead', DeprecationWarning)
    if enabled:
        return enable_file_sync()
    else:
        return disable_file_sync()


def connect_file_sync():
    """Connect the file sync service."""
    return get_syncdaemon_tool().connect()


def disconnect_file_sync():
    """Disconnect the file sync service."""
    return get_syncdaemon_tool().disconnect()


def start_file_sync():
    """Start the file sync service."""
    return get_syncdaemon_tool().start()


def stop_file_sync():
    """Stop the file sync service."""
    return get_syncdaemon_tool().quit()
